/***************************************************************************
 *   Copyright (C) 2005 by Emiliano Gulmini   *
 *   emi_barbarossa@yahoo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//QT headers
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qdatetime.h>
#include <qlayout.h>
//KDE headers
#include <kapplication.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstdguiitem.h>
#include <kpushbutton.h>
#include <karrowbutton.h>
#include <kurlrequester.h>
#include <khelpmenu.h>
//Local headers
#include "kinittabeditor.h"

KInittabEditor::KInittabEditor()
    : QInittabEditor( 0, "KInittabEditor" )
{
    // set the shell's ui resource file
    //setXMLFile("kinittabeditorui.rc");
  /** Current date */
  m_creationDate = QDate::currentDate(Qt::LocalTime).toString(Qt::LocalDate);
  /** Connecting dialog buttons */
  connect(m_pbOk, SIGNAL(clicked()), this, SLOT(createInittabFile()));
  connect(m_pbCancel, SIGNAL(clicked()), this, SLOT(reject()));
  /** Connecting add/delete buttons */
  connect(m_pbAddEntry, SIGNAL(clicked()), this, SLOT(addEntry()));
  connect(m_pbDelEntry, SIGNAL(clicked()), this ,SLOT(delEntry()));
  /** Connecting general informations widgets */
  connect(m_leAuthor, SIGNAL(textChanged(const QString&)), this, SLOT(getAuthor(const QString&)));
  connect(m_leVersion, SIGNAL(textChanged(const QString&)), this, SLOT(getVersion(const QString&)));
  connect(m_kurlreqPath, SIGNAL(textChanged(const QString&)), this, SLOT(getInittabPath(const QString&)));
  /** Connecting entries widgets */
  connect(m_leID, SIGNAL(textChanged(const QString&)), this, SLOT(getID(const QString&)));
  connect(m_leRunLevel, SIGNAL(textChanged(const QString&)), this, SLOT(getRunLevel(const QString&)));
  connect(m_leAction, SIGNAL(textChanged(const QString&)), this, SLOT(getAction(const QString&)));
  connect(m_leProcess, SIGNAL(textChanged(const QString&)), this, SLOT(getProcess(const QString&)));
  /** Connecting comment widgets */
  connect(m_teComment, SIGNAL(textChanged()), this, SLOT(getComment()));
  /** Connecting help */
  connect(m_pbHelp, SIGNAL(clicked()), this, SLOT(helpButton()));
  KHelpMenu *m_helpMenu = new KHelpMenu(this, KGlobal::instance()->aboutData(), true);
  m_pbHelp->setPopup((QPopupMenu*)m_helpMenu->menu());
}

KInittabEditor::~KInittabEditor()
{
}

/*!
    \fn KInittabEditor::addEntry
 */
void KInittabEditor::addEntry()
{
  /** Creating an entry in the list box */
  QString entry = QString("%1:%2:%3:%4  [%5]").arg(m_ID)
                                              .arg(m_runLevel)
                                              .arg(m_action)
                                              .arg(m_process)
                                              .arg(m_comment);
  /** Avoiding empty entries */
  if(entry != ":::  []")
    m_lbEntries->insertItem(entry);
  /** Cleaning input widgets */
  m_leID->clear();
  m_leRunLevel->clear();
  m_leAction->clear();
  m_leProcess->clear();
  m_teComment->clear();
}

/*!
    \fn KInittabEditor::delEntry
 */
void KInittabEditor::delEntry()
{
  QString currentEntry = m_lbEntries->currentText();
  if(currentEntry.isEmpty()) return;
  /** Extracting entry fields */
  m_leID->setText(currentEntry.section(":",0,0));
  m_leRunLevel->setText(currentEntry.section(":",1,1));
  m_leAction->setText(currentEntry.section(":",2,2));
  m_leProcess->setText(currentEntry.section(":",3,3).section("  ",0,0));
  m_teComment->setText(currentEntry.section(QRegExp("(\\s+\\[|\\])"),1,1));
  /** Cleaning list */
  m_lbEntries->removeItem(m_lbEntries->currentItem());
}

/*!
    \fn KInittabEditor::getID(const QString&)
 */
void KInittabEditor::getID(const QString& ID)
{
  m_ID = ID;
}

/*!
    \fn KInittabEditor::getRunLevel(const QString&)
 */
void KInittabEditor::getRunLevel(const QString& runLevel)
{
  m_runLevel = runLevel;
}

/*!
    \fn KInittabEditor::getAction(const QString&)
 */
void KInittabEditor::getAction(const QString& action)
{
  m_action = action;
}

/*!
    \fn KInittabEditor::getProcess(const QString&)
 */
void KInittabEditor::getProcess(const QString& process)
{
  m_process = process;
}

/*!
    \fn KInittabEditor::getComment()
 */
void KInittabEditor::getComment()
{
  m_comment = m_teComment->text();
}

/*!
    \fn KInittabEditor::getAuthor(const QString&)
 */
void KInittabEditor::getAuthor(const QString& author)
{
  m_author = author;
}

/*!
    \fn KInittabEditor::getVersion(const QString&)
 */
void KInittabEditor::getVersion(const QString& version)
{
  m_version = version;
}

/*!
    \fn KInittabEditor::getInittabPath(const QString& path)
 */
void KInittabEditor::getInittabPath(const QString& path)
{
  m_inittabPath = path;
}

/*!
    \fn KInittabEditor::createInittabFile()
 */
void KInittabEditor::createInittabFile()
{
  QFileInfo inittabFile(m_inittabPath);
  if(inittabFile.exists())
    {
      /** First saving old inittab file */
      QDir d;
      if(not d.rename(m_inittabPath,
                      m_inittabPath+".orig",
                      true))
        {
          KMessageBox::error(0,"You have no permissions.");
          return;
        }
      else
        {
          KMessageBox::information(0,i18n("Your old file has been renamed in %1.orig").arg(m_inittabPath));
        }
      /** Then creating new inittab file */
      QFile newInittab(m_inittabPath);
      newInittab.open(IO_WriteOnly);
      QTextStream inittabStream(&newInittab);
      QString inittabContent = "# INITTAB FILE\n\n";
              inittabContent += "# "+i18n("This file is used to setup the system in a specific run-level")+"\n\n";
              inittabContent += "# "+i18n("Author: %1").arg(m_author)+"\n";
              inittabContent += "# "+i18n("Version: %1").arg(m_version)+"\n";
              inittabContent += "# "+i18n("Date: %1").arg(m_creationDate)+"\n";
              inittabContent += "# "+i18n("Created with KInittabEditor written by Emiliano Gulmini (e-mail:emi_barbarossa AT yahoo.it) under G.P.L.")+"\n\n";
      /** Creating the entries */
      int entriesNumber = m_lbEntries->numRows(),
          i;
      for(i = 0; i < entriesNumber; i++)
        {
          QRegExp sep( "(\\s+\\[|\\])" );
          QListBoxItem* lbi = m_lbEntries->item(i);
          QString entryString = lbi->text().section(sep,0,0);
          QString commentString = lbi->text().section(sep,1,1);
          //if(entryString.isEmpty())
          inittabContent += "#"+commentString+"\n";
          inittabContent += entryString+"\n\n";
        }
      inittabContent += "# End of inittab file";
      /** Adding the generated content to the file */
      inittabStream << inittabContent;
      /** Now the file is ready */
      newInittab.close();
    }
  else
    {
      KMessageBox::error(0, i18n("The file inittab was not found! Maybe that your OS uses a different initialization method"));
    }

  accept();
}

/*!
    \fn KInittabEditor::helpButton()
 */
void KInittabEditor::helpButton()
{
  kapp->invokeHelp();
}
#include "kinittabeditor.moc"

